@@ -44,10 +44,12 @@ is only applied once, and applies to everything subsequently compiled.
 This module is implemented in XS, with a pure Perl backup version for
 systems that can't handle XS modules.  The XS version has a better chance
 of playing nicely with other modules that modify C<require> handling.
-The pure Perl version can't work at all on some Perl versions; users of
-those versions must use the XS.
+The pure Perl version can't work at all on some Perl versions; users
+of those versions must use the XS.  On all Perl versions suffering the
+underlying hint leakage bug, pure Perl hooking of C<require> breaks the
+use of C<require> without an explicit parameter (implicitly using C<$_>).
 
-=head1 PERL VERION DIFFERENCES
+=head1 PERL VERSION DIFFERENCES
 
 The history of the C<%^H> bugs is complex.  Here is a chronological
 statement of the relevant changes.
@@ -143,7 +145,7 @@ BEGIN { ${^WARNING_BITS} = ""; }
 # Also don't "use strict", because of consequences of compiling
 # strict.pm's code.
 
-our $VERSION = "0.007";
+our $VERSION = "0.008";
 
 if("$]" >= 5.012) {
 	# bug not present
@@ -161,9 +163,13 @@ if("$]" >= 5.012) {
 	# may be subject to delayed require statements in XSLoader or
 	# things that it loaded.
 	foreach(qw(Carp.pm Carp/Heavy.pm)) {
-		eval { local $SIG{__DIE__}; require($_); };
+		eval { local $SIG{__DIE__}; require($_); 1; };
 	}
-} elsif("$]" >= 5.007002 && !("$]" >= 5.009004 && "$]" < 5.010001)) {
+} elsif("$]" < 5.007002) {
+	die "pure Perl version of @{[__PACKAGE__]} can't work on pre-5.8 perl";
+} elsif("$]" >= 5.009004 && "$]" < 5.010001) {
+	die "pure Perl version of @{[__PACKAGE__]} can't work on perl 5.10.0";
+} else {
 	my $done;
 	*import = sub {
 		die "$_[0] does not take any importation arguments\n"
@@ -189,15 +195,36 @@ if("$]" >= 5.012) {
 				$CORE::GLOBAL::{require} = $grequire;
 				return scalar($requirer->($arg));
 			};
-		*CORE::GLOBAL::require = sub {
+		*CORE::GLOBAL::require = sub ($) {
 			die "wrong number of arguments to require\n"
 				unless @_ == 1;
 			my($arg) = @_;
 			# Some reference to $next_require is required
 			# at this level of subroutine so that it will
 			# be closed over and hence made available to
-			# the string eval below.
+			# the string eval.
 			my $nr = $next_require;
+			my $requirer = eval qq{
+				package @{[scalar(caller(0))]};
+				sub { scalar(\$next_require->(\$_[0])) };
+			};
+			# We must localise %^H when performing a require
+			# with a filename, but not a require with a
+			# version number.  This is because on Perl 5.9.5
+			# and above require with a version number does an
+			# internal importation from the "feature" module,
+			# which is intentional behaviour that must be
+			# allowed to affect %^H.  (That's logically the
+			# wrong place for the feature importation, but
+			# it's too late to change how old Perls do it.)
+			# A version number is an argument that is either
+			# numeric or, from Perl 5.9.2 onwards, a v-string.
+			my $must_localise = ($arg^$arg) ne "0" &&
+				!("$]" >= 5.009002 && ref(\$arg) eq "VSTRING");
+			# On Perl 5.11 we need to set the HINT_LOCALIZE_HH
+			# bit to get proper restoration of %^H by the
+			# swash loading code.
+			$^H |= 0x20000 if "$]" >= 5.011 && $must_localise;
 			# Compile-time %^H gets localised by the
 			# "local %^H".	Runtime %^H doesn't exist prior
 			# to Perl 5.9.4, and on Perl 5.10.1 and above is
@@ -206,21 +233,12 @@ if("$]" >= 5.012) {
 			# correctly localise runtime %^H in pure Perl,
 			# short of putting an eval frame around the
 			# require, so we don't use this implementation in
-			# that region.	On Perl 5.11 we need to set the
-			# HINT_LOCALIZE_HH bit to get proper restoration
-			# of %^H by the swash loading code.
-			my $requirer = eval qq{
-				package @{[scalar(caller(0))]};
-				sub { scalar(\$next_require->(\$_[0])) };
-			};
-			$^H |= 0x20000 if "$]" >= 5.011;
-			local %^H;
+			# that region.
+			local %^H if $must_localise;
 			return scalar($requirer->($arg));
 		};
 	};
 	*unimport = sub { die "$_[0] does not support unimportation\n" };
-} else {
-	die "pure Perl version of @{[__PACKAGE__]} can't work on pre-5.8 perl";
 }
 
 =head1 BUGS
@@ -230,6 +248,18 @@ C<require>.  As a result, it cannot prevent lexical state leakage through
 a C<require> statement that was compiled before this module was invoked.
 Where problems occur, this module must be invoked earlier.
 
+On all Perl versions that need a fix for the lexical hint leakage bug,
+the pure Perl implementation of this module unavoidably breaks the use
+of C<require> without an explicit parameter (implicitly using C<$_>).
+This is due to another bug in the Perl core, fixed in Perl 5.15.5, and is
+inherent to the mechanism by which pure Perl code can hook C<require>.
+The use of implicit C<$_> with C<require> is rare, so although this
+state of affairs is faulty it will actually work for most programs.
+Perl versions 5.12.0 and greater, despite having the C<require> hooking
+bug, don't actually exhibit a problem with the pure Perl version of this
+module, because with the lexical hint leakage bug fixed there is no need
+for this module to hook C<require>.
+
 =head1 SEE ALSO
 
 L<perlpragma>
@@ -240,7 +270,7 @@ Andrew Main (Zefram) <zefram@fysh.org>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2009, 2010, 2011, 2012
+Copyright (C) 2009, 2010, 2011, 2012, 2015
 Andrew Main (Zefram) <zefram@fysh.org>
 
 =head1 LICENSE
